// 
// Copyright (C) 2014, NorthTrader
// V1  Nov-2014  NT  Initial version
//
// Change version number in Description too!

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// TZPriceAnalysis - analysis of Transient Zones (TZs) by price level
    /// </summary>
    [Description("This version (V1) plots historical Transient Zones (zones that price doesn't hit within h bars left and right of middle bar) " +
		"on the chart and provides statistics by price level for the given price region.\n" + 
		"\"RZ\" = h-recurrent zone\n" +
		"\"TZ\" = h-transient zone\n" + 
		"TZs which appear immediately after another TZ are a different color.\n" +
		"See the Output window for statistics by price level."  
	)]
    public class TZPriceAnalysis : Indicator
    {
        #region Variables

		//Parameters 
		private int iH_Left = 20;         //hLeft
		private int iH_Right = 20;        //hRight
		private double dPriceLow = 1.2;   //PriceLow
		private double dPriceHigh = 1.4;  //PriceHigh

		//Plot colours 
		private Color colTZ = Color.Blue;
		private Color colConsecTZ = Color.Red;

		//Other 
		private	int iHRange, iBarMid, iBarFirst, iBarLast;
		private	int iLastP, iTotalBars;
		private double dActualPriceLow, dActualPriceHigh;
		private int[] iRZAfterRZS, iTZAfterRZS, iRZAfterTZS, iTZAfterTZS;
		private double[] dPriceS;
		private bool[] bLastTransientS;
		
        #endregion

        protected override void Initialize()
        {
			//Initialize variables
			iHRange = hLeft + hRight;
			iBarMid = hRight;
			iBarFirst = iBarMid + hLeft;
			iBarLast = 0;  //iBarMid - hLeft;
			iTotalBars = 0;
			dActualPriceLow = dActualPriceHigh = 0;

			DisplayInDataBox	= false;
			PaintPriceMarkers	= false;
            Overlay				= true;
			CalculateOnBarClose = true;	 //Not updated on every tick

        }

		protected override void OnBarUpdate()
        {
			if (CurrentBar == 0) SetVars();
			if (CurrentBar < iHRange) return;

			iTotalBars++;

			//Loop through all prices in region
			for (int liP = 0; liP <= iLastP; liP++)
			{
				
				//Check for price within middle bar
				if (Low[iBarMid] <= dPriceS[liP] && High[iBarMid] >= dPriceS[liP])
				{
					//New zone - check for recurrence
					bool lbTransient = true;
					//Loop from last bar of zone to first
					for (int liBar = iBarLast; liBar <= iBarFirst; liBar++)
					{	
						if (liBar == iBarMid) continue;  //Skip middle bar
						if (Low[liBar] < dPriceS[liP] && High[liBar] >= dPriceS[liP])
						{
							//Recurrent zone
							lbTransient = false;
							break;
						}
					}
					
					if (lbTransient)
					{
						Color lcolTZ;
						if (bLastTransientS[liP])
						{
							iTZAfterTZS[liP]++;
							lcolTZ = ConsecutiveTZColor;
						}
						else
						{
							iTZAfterRZS[liP]++;
							lcolTZ = TZColor;
						}
						DrawRectangle("TZ" + CurrentBar + "-" + liP, true, iBarFirst, dPriceS[liP], iBarLast, dPriceS[liP] - TickSize, Color.Empty, lcolTZ, 4);
					}
					else
					{
						if (bLastTransientS[liP]) iRZAfterTZS[liP]++;
						else iRZAfterRZS[liP]++;
					}
					bLastTransientS[liP] = lbTransient;
				}
			}
			
			//Process and display stats on last bar
			if (CurrentBar == Count - 2) ProcessStats();		
		
		}

        #region Functions

		private void ProcessStats()
		{
			//Called on last bar
			string lsSep = ", ";
			int liTotalZones, liTotalRZs, liTotalTZs, liTotalZonesAfterTZs;
			int liGrandTotalZones = 0;
			int liGrandTotalRZs = 0;
			int liGrandTotalTZs = 0;
			int liGrandTotalRZAfterTZs = 0;
			int liGrandTotalZonesAfterTZs = 0;

			Print(DateTime.Now.ToString());
			Print("h Left = " + hLeft + lsSep + "h Right = " + hRight + lsSep + "Pip = " + TickSize.ToString() + lsSep +
				"Bar = " + BarsPeriod.Value + " " + BarsPeriod.Id);
			Print("Given price region = " + PriceLow.ToString() + " to " + PriceHigh.ToString());
			Print("Total bars = " + iTotalBars);
			for (int liP = 0; liP <= iLastP; liP++)
			{
				liTotalRZs = iRZAfterRZS[liP] + iRZAfterTZS[liP];
				liTotalTZs = iTZAfterRZS[liP] + iTZAfterTZS[liP];
				liTotalZones = liTotalRZs + liTotalTZs;
				if (liTotalZones > 0) 
				{
					//Store first active price level 
					if (dActualPriceLow == 0) dActualPriceLow = dPriceS[liP];
					
					liTotalZonesAfterTZs = iRZAfterTZS[liP] + iTZAfterTZS[liP];
					Print("  Price = " + dPriceS[liP].ToString() + " : " + "TotalZs = " + liTotalZones + lsSep +
						"RZs = " + liTotalRZs + " (" + ((double)liTotalRZs / liTotalZones).ToString("0.0%") + ")" + lsSep +
						"TZs = " + liTotalTZs + " (" + ((double)liTotalTZs / liTotalZones).ToString("0.0%") + ")" +
						((liTotalZonesAfterTZs > 0) ? lsSep + "RZs after TZs = " + ((double)iRZAfterTZS[liP] / liTotalZonesAfterTZs).ToString("0.0%") : ""));
					liGrandTotalZones += liTotalZones;
					liGrandTotalRZs += liTotalRZs;
					liGrandTotalTZs += liTotalTZs;
					liGrandTotalRZAfterTZs += iRZAfterTZS[liP];
					liGrandTotalZonesAfterTZs += liTotalZonesAfterTZs;
				}
				else
				{
					//Store last active price level (assumes no continuous gap)
					if (dActualPriceLow != 0 && dActualPriceHigh == 0) dActualPriceHigh = dPriceS[liP - 1];
				}
			}
			if (dActualPriceHigh == 0) dActualPriceHigh = dPriceS[iLastP];
			Print("Actual price region = " + dActualPriceLow.ToString() + " to " + dActualPriceHigh.ToString());
			string lsRZs = liGrandTotalRZs + " (" + ((double)liGrandTotalRZs / liGrandTotalZones).ToString("0.0%") + ")";
			string lsRZsAfterTZs = ((double)liGrandTotalRZAfterTZs / liGrandTotalZonesAfterTZs).ToString("0.0%") +
				"(" + liGrandTotalRZAfterTZs + " / " + liGrandTotalZonesAfterTZs + ")";
			Print("Total zones = " + liGrandTotalZones + lsSep + 
				"Total RZs = " + lsRZs + lsSep +
				"Total RZs after TZs = " + lsRZsAfterTZs);
			
			string lsStats = "h=" + hLeft + "/" + hRight + lsSep;
			lsStats += "Prices=" + dActualPriceLow.ToString() + "-" + dActualPriceHigh.ToString() + lsSep;
			lsStats += "Pip=" + TickSize.ToString() + lsSep;
			lsStats += "Bars=" + iTotalBars + lsSep;
			lsStats += "Zones=" + liGrandTotalZones + lsSep;
			lsStats += "RZs=" + lsRZs + lsSep;
			lsStats += "RZs after TZs=" + lsRZsAfterTZs;
			DrawTextFixed("stats", lsStats, TextPosition.TopRight);

		}
		
		private void SetVars()
		{
			//Called on first bar because some properties unavailable during Initialize()   
			ClearOutputWindow();
			iLastP = (int)Math.Round((PriceHigh - PriceLow) / TickSize, 0);
			dPriceS = new double[iLastP + 1];
			iRZAfterRZS = new int[iLastP + 1];
			iTZAfterRZS = new int[iLastP + 1];
			iRZAfterTZS = new int[iLastP + 1];
			iTZAfterTZS = new int[iLastP + 1];
			bLastTransientS = new bool[iLastP + 1];
			int liX = 0;
			for (double ldPrice = PriceLow; ldPrice <= PriceHigh; ldPrice += TickSize)
			{
				dPriceS[liX] = Math.Round(ldPrice,4);
				iRZAfterRZS[liX] = iTZAfterRZS[liX] = iRZAfterTZS[liX] = iTZAfterTZS[liX] = 0;
				bLastTransientS[liX] = false;
				liX++;
			}
		}
		
		#endregion

        #region Properties
		
		[Description("Number of bars to the left of middle bar")]
        [GridCategory("Parameters")]
        [Gui.Design.DisplayName("h Left")]
        public int hLeft
        {
            get { return iH_Left; }
            set { iH_Left = Math.Max(1, value); }
        }
		[Description("Number of bars to the right of middle bar")]
        [GridCategory("Parameters")]
        [Gui.Design.DisplayName("h Right")]
        public int hRight
        {
            get { return iH_Right; }
            set { iH_Right = Math.Max(1, value); }
        }
		[Description("Lowest price in analysis area")]
        [GridCategory("Parameters")]
        [Gui.Design.DisplayName("Price Low")]
        public double PriceLow
        {
            get { return dPriceLow; }
            set { dPriceLow = Math.Min(dPriceHigh, value); }
        }
		
		[Description("Highest price in analysis area")]
        [GridCategory("Parameters")]
        [Gui.Design.DisplayName("Price High")]
        public double PriceHigh
        {
            get { return dPriceHigh; }
            set { dPriceHigh = Math.Max(dPriceLow, value); }
        }

		[XmlIgnore]
		[Description("Color of h-transient zone")]
        [GridCategory("Plots")]
        [Gui.Design.DisplayName("TZ Color")]
		public Color TZColor
        {
            get { return colTZ; }
            set { colTZ = value; }
        }
		//Serialize Color object
		[Browsable(false)]
		public string TZColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(colTZ); }
			set { colTZ = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		[XmlIgnore]
		[Description("Color of consecutive h-transient zone")]
        [GridCategory("Plots")]
        [Gui.Design.DisplayName("Consec TZ Color")]
        public Color ConsecutiveTZColor
        {
            get { return colConsecTZ; }
            set { colConsecTZ = value; }
        }
		//Serialize Color object
		[Browsable(false)]
		public string ConsecTZColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(colConsecTZ); }
			set { colConsecTZ = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		#endregion
    }
}
